//////////////////////////////////////////////////////////////////////////////
// Filename:          /home/mmetzner/Testing/Xilinx/s3a_vga/drivers/plb_raw_master_v1_00_a/src/plb_raw_master.c
// Version:           1.00.a
// Description:       plb_raw_master Driver Source File
// Date:              Thu Jan 29 10:29:46 2009 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////


/***************************** Include Files *******************************/

#include "plb_raw_master.h"

/************************** Function Definitions ***************************/


/**
 *
 * User logic master module to send/receive bytes to/from remote system memory.
 * While sending, the bytes are read from user logic local FIFO and write to remote system memory.
 * While receiving, the bytes are read from remote system memory and write to user logic local FIFO.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 * @param   DstAddress is the destination system memory address from/to which the data will be fetched/stored.
 * @param   Size is the number of bytes to be sent.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void PLB_RAW_MASTER_MasterSendByte(Xuint32 BaseAddress, Xuint32 DstAddress, int Size)
{
  /*
   * Set user logic master control register for write transfer.
   */
  XIo_Out8(BaseAddress+PLB_RAW_MASTER_MST_CNTL_REG_OFFSET, MST_SGWR);

  /*
   * Set user logic master address register to drive IP2Bus_Mst_Addr signal.
   */
  XIo_Out32(BaseAddress+PLB_RAW_MASTER_MST_ADDR_REG_OFFSET, DstAddress);

  /*
   * Set user logic master byte enable register to drive IP2Bus_Mst_BE signal.
   */
  XIo_Out16(BaseAddress+PLB_RAW_MASTER_MST_BE_REG_OFFSET, 0xFFFF);

  /*
   * Set user logic master length register.
   */
  XIo_Out16(BaseAddress+PLB_RAW_MASTER_MST_LEN_REG_OFFSET, (Xuint16) Size);
  /*
   * Start user logic master write transfer by writting special pattern to its go port.
   */
  XIo_Out8(BaseAddress+PLB_RAW_MASTER_MST_GO_PORT_OFFSET, MST_START);
}

void PLB_RAW_MASTER_MasterRecvByte(Xuint32 BaseAddress, Xuint32 DstAddress, int Size)
{
  /*
   * Set user logic master control register for read transfer.
   */
  XIo_Out8(BaseAddress+PLB_RAW_MASTER_MST_CNTL_REG_OFFSET, MST_SGRD);

  /*
   * Set user logic master address register to drive IP2Bus_Mst_Addr signal.
   */
  XIo_Out32(BaseAddress+PLB_RAW_MASTER_MST_ADDR_REG_OFFSET, DstAddress);

  /*
   * Set user logic master byte enable register to drive IP2Bus_Mst_BE signal.
   */
  XIo_Out16(BaseAddress+PLB_RAW_MASTER_MST_BE_REG_OFFSET, 0xFFFF);

  /*
   * Set user logic master length register.
   */
  XIo_Out16(BaseAddress+PLB_RAW_MASTER_MST_LEN_REG_OFFSET, (Xuint16) Size);
  /*
   * Start user logic master read transfer by writting special pattern to its go port.
   */
  XIo_Out8(BaseAddress+PLB_RAW_MASTER_MST_GO_PORT_OFFSET, MST_START);
}

