//////////////////////////////////////////////////////////////////////////////
// Filename:          /home/mmetzner/Testing/Xilinx/s3a_vga/drivers/plb_raw_master_v1_00_a/src/plb_raw_master.h
// Version:           1.00.a
// Description:       plb_raw_master Driver Header File
// Date:              Thu Jan 29 10:29:46 2009 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////

#ifndef PLB_RAW_MASTER_H
#define PLB_RAW_MASTER_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions ***************************/


/**
 * User Logic Slave Space Offsets
 * -- SLV_REG0 : user logic slave module register 0
 * -- SLV_REG1 : user logic slave module register 1
 */
#define PLB_RAW_MASTER_USER_SLV_SPACE_OFFSET (0x00000000)
#define PLB_RAW_MASTER_SLV_REG0_OFFSET (PLB_RAW_MASTER_USER_SLV_SPACE_OFFSET + 0x00000000)
#define PLB_RAW_MASTER_SLV_REG1_OFFSET (PLB_RAW_MASTER_USER_SLV_SPACE_OFFSET + 0x00000004)

/**
 * User Logic Master Space Offsets
 * -- MST_CNTL_REG : user logic master module control register
 * -- MST_STAT_REG : user logic master module status register
 * -- MST_ADDR_REG : user logic master module address register
 * -- MST_BE_REG   : user logic master module byte enable register
 * -- MST_LEN_REG  : user logic master module length (data transfer in bytes) register
 * -- MST_GO_PORT  : user logic master module go bit (to start master operation)
 */
#define PLB_RAW_MASTER_USER_MST_SPACE_OFFSET (0x00000100)
#define PLB_RAW_MASTER_MST_CNTL_REG_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x00000000)
#define PLB_RAW_MASTER_MST_STAT_REG_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x00000001)
#define PLB_RAW_MASTER_MST_ADDR_REG_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x00000004)
#define PLB_RAW_MASTER_MST_BE_REG_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x00000008)
#define PLB_RAW_MASTER_MST_LEN_REG_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x0000000C)
#define PLB_RAW_MASTER_MST_GO_PORT_OFFSET (PLB_RAW_MASTER_USER_MST_SPACE_OFFSET + 0x0000000F)

/**
 * User Logic Master Module Masks
 * -- MST_RD_MASK   : user logic master read request control
 * -- MST_WR_MASK   : user logic master write request control
 * -- MST_BL_MASK   : user logic master bus lock control
 * -- MST_BRST_MASK : user logic master burst assertion control
 * -- MST_DONE_MASK : user logic master transfer done status
 * -- MST_BSY_MASK  : user logic master busy status
 * -- MST_BRRD      : user logic master burst read request
 * -- MST_BRWR      : user logic master burst write request
 * -- MST_SGRD      : user logic master single read request
 * -- MST_SGWR      : user logic master single write request
 * -- MST_START     : user logic master to start transfer
 */
#define MST_RD_MASK (0x80000000UL)
#define MST_WR_MASK (0x40000000UL)
#define MST_BL_MASK (0x20000000UL)
#define MST_BRST_MASK (0x10000000UL)
#define MST_DONE_MASK (0x00800000UL)
#define MST_BSY_MASK (0x00400000UL)
#define MST_BRRD (0x90)
#define MST_BRWR (0x50)
#define MST_SGRD (0x80)
#define MST_SGWR (0x40)
#define MST_START (0x0A)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a PLB_RAW_MASTER register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void PLB_RAW_MASTER_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define PLB_RAW_MASTER_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a PLB_RAW_MASTER register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 PLB_RAW_MASTER_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define PLB_RAW_MASTER_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Write/Read 32 bit value to/from PLB_RAW_MASTER user logic slave registers.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 * @param   RegOffset is the offset from the slave register to write to or read from.
 * @param   Value is the data written to the register.
 *
 * @return  Data is the data from the user logic slave register.
 *
 * @note
 * C-style signature:
 * 	void PLB_RAW_MASTER_mWriteSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Value)
 * 	Xuint32 PLB_RAW_MASTER_mReadSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define PLB_RAW_MASTER_mWriteSlaveReg0(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (PLB_RAW_MASTER_SLV_REG0_OFFSET) + (RegOffset), (Xuint32)(Value))
#define PLB_RAW_MASTER_mWriteSlaveReg1(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (PLB_RAW_MASTER_SLV_REG1_OFFSET) + (RegOffset), (Xuint32)(Value))

#define PLB_RAW_MASTER_mReadSlaveReg0(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (PLB_RAW_MASTER_SLV_REG0_OFFSET) + (RegOffset))
#define PLB_RAW_MASTER_mReadSlaveReg1(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (PLB_RAW_MASTER_SLV_REG1_OFFSET) + (RegOffset))

/**
 *
 * Check status of PLB_RAW_MASTER user logic master module.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 *
 * @return  Status is the result of status checking.
 *
 * @note
 * C-style signature:
 * 	bool PLB_RAW_MASTER_mMasterDone(Xuint32 BaseAddress)
 * 	bool PLB_RAW_MASTER_mMasterBusy(Xuint32 BaseAddress)
 * 	bool PLB_RAW_MASTER_mMasterError(Xuint32 BaseAddress)
 * 	bool PLB_RAW_MASTER_mMasterTimeout(Xuint32 BaseAddress)
 *
 */
#define PLB_RAW_MASTER_mMasterDone(BaseAddress) \
 	((((Xuint32) XIo_In8((BaseAddress)+(PLB_RAW_MASTER_MST_STAT_REG_OFFSET)))<<16 & MST_DONE_MASK) == MST_DONE_MASK)
#define PLB_RAW_MASTER_mMasterBusy(BaseAddress) \
 	((((Xuint32) XIo_In8((BaseAddress)+(PLB_RAW_MASTER_MST_STAT_REG_OFFSET)))<<16 & MST_BUSY_MASK) == MST_BUSY_MASK)
#define PLB_RAW_MASTER_mMasterError(BaseAddress) \
 	((((Xuint32) XIo_In8((BaseAddress)+(PLB_RAW_MASTER_MST_STAT_REG_OFFSET)))<<16 & MST_ERROR_MASK) == MST_ERROR_MASK)
#define PLB_RAW_MASTER_mMasterTimeout(BaseAddress) \
 	((((Xuint32) XIo_In8((BaseAddress)+(PLB_RAW_MASTER_MST_STAT_REG_OFFSET)))<<16 & MST_TIMEOUT_MASK) == MST_TIMEOUT_MASK)

/************************** Function Prototypes ****************************/


/**
 *
 * User logic master module to send/receive bytes to/from remote system memory.
 * While sending, the bytes are read from user logic local FIFO and write to remote memory.
 * While receiving, the bytes are read from remote memory and write to user logic local FIFO.
 *
 * @param   BaseAddress is the base address of the PLB_RAW_MASTER device.
 * @param   DstAddress is the destination memory address from/to which the data will be fetched/stored.
 * @param   Size is the number of bytes to be sent.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void PLB_RAW_MASTER_MasterSendByte(Xuint32 BaseAddress, Xuint32 DstAddress, int Size);
void PLB_RAW_MASTER_MasterRecvByte(Xuint32 BaseAddress, Xuint32 DstAddress, int Size);

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the PLB_RAW_MASTER instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus PLB_RAW_MASTER_SelfTest(void * baseaddr_p);

#endif // PLB_RAW_MASTER_H
