-----------------------------------------------------------------------------------------------------------------------
-- FILE			: line_buffer.vhd
-- TITLE		:
-- PROJECT		:
-- AUTHOR		: Lucio
-- DESCRIPTION	: Implements a 16bit to 24bit-depth conversor and a 640 24bit-depth pixel buffer
-- NOTES		:
-- REVISION		: $Id$
-- DEPENDENCIES	: To be used within vga_out_16bit module
-----------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------------------------------------------------------------
library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_unsigned.all;
	use ieee.std_logic_arith.all;

-----------------------------------------------------------------------------------------------------------------------
-- Ports                                                                                                                					-- Entidade --
-----------------------------------------------------------------------------------------------------------------------
entity line_buffer is
port
(
	-- Clocks and resets -----------------------------------------------------------------------------------------------
	reset					: in	std_logic;						-- rst signal
	pixel_clk			: in	std_logic;						-- pixel clock - 25MHz
	plb_clk				: in	std_logic;						-- plb clock
	color_depth		: in	std_logic_vector(1 downto 0);	-- Color depth
	-- Write signals ---------------------------------------------------------------------------------------------------
	wr_pixel_col	: in	std_logic_vector(9 downto 0);	-- write address
	wr_pixel_val	: in	std_logic_vector(63 downto 0);	-- write value
	wr_enable			: in	std_logic;						-- write enable
	-- Read signals ----------------------------------------------------------------------------------------------------
	rd_pixel_col	: in	std_logic_vector(9 downto 0);	-- pixel column
	rd_pixel			: in	std_logic;						-- read enable
	-- To VGA ----------------------------------------------------------------------------------------------------------
	red				: out	std_logic_vector(7 downto 0);	-- red output
	green			: out	std_logic_vector(7 downto 0);	-- green output
	blue			: out	std_logic_vector(7 downto 0)	-- blue output
);
end line_buffer;

-----------------------------------------------------------------------------------------------------------------------
-- Architecture
-----------------------------------------------------------------------------------------------------------------------
architecture behavior of  line_buffer is

	function invert_8(dt : std_logic_vector) return std_logic_vector is
	--variable result	: std_logic_vector(dt'length - 1 downto 0);
	variable result	: std_logic_vector(dt'high downto dt'low);
	begin
		for i in dt'low to dt'high loop
			result(i)	:=	dt(dt'high - i + dt'low);
		end loop;
		return result;
	end function invert_8;

	-- -------------------------------
	-- Signal Declarations
	-- -------------------------------
	type	ram_type		is array (159 downto 0) of std_logic_vector (63 downto 0);
	signal	pixel_buffer	: ram_type := ( others => (x"0000000000000000") );
	signal	inv_wr_val		: std_logic_vector(63 downto 0);

	-- -------------------------------
	-- Components
	-- -------------------------------

begin

	process(plb_clk)
	begin
		if (plb_clk'event and plb_clk = '1') then
			if (wr_enable = '1') then
				pixel_buffer(conv_integer(wr_pixel_col))	<= inv_wr_val; --wr_pixel_val;
			end if;
		end if;
	end process;

	process(pixel_clk)
	begin
		if pixel_clk'event and pixel_clk = '0' then
			if color_depth = "11" then
				if rd_pixel = '1' then
					if rd_pixel_col(0) = '0' then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(7 downto 0));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(15 downto 8));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(23 downto 16));
					elsif rd_pixel_col(0) = '1' then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(39 downto 32));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(47 downto 40));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 1)))(55 downto 48));
					end if;
				else
					red		<=	(others => '0');
					green	<=	(others => '0');
					blue	<=	(others => '0');
				end if;
			end if;
			if color_depth = "10" then
				if rd_pixel = '1' then
					if rd_pixel_col(1 downto 0) = "00" then
						red		<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(4 downto 0));
						green	<=	invert_8("00" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(10 downto 5));
						blue	<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(15 downto 11));
					elsif rd_pixel_col(1 downto 0) = "01" then
						red		<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(20 downto 16));
						green	<=	invert_8("00" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(26 downto 21));
						blue	<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(31 downto 27));
					elsif rd_pixel_col(1 downto 0) = "10" then
						red		<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(36 downto 32));
						green	<=	invert_8("00" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(42 downto 37));
						blue	<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(47 downto 43));
					elsif rd_pixel_col(1 downto 0) = "11" then
						red		<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(52 downto 48));
						green	<=	invert_8("00" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(58 downto 53));
						blue	<=	invert_8("000" & pixel_buffer(conv_integer(rd_pixel_col(9 downto 2)))(63 downto 59));
					end if;
				else
					red		<=	(others => '0');
					green	<=	(others => '0');
					blue	<=	(others => '0');
				end if;
			end if;
			if color_depth = "00" then
				if rd_pixel = '1' then
					if rd_pixel_col(2 downto 0) = "000" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(7 downto 0));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(7 downto 0));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(7 downto 0));
					elsif rd_pixel_col(2 downto 0) = "001" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(15 downto 8));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(15 downto 8));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(15 downto 8));
					elsif rd_pixel_col(2 downto 0) = "010" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(23 downto 16));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(23 downto 16));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(23 downto 16));
					elsif rd_pixel_col(2 downto 0) = "011" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(31 downto 24));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(31 downto 24));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(31 downto 24));
					elsif rd_pixel_col(2 downto 0) = "100" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(39 downto 32));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(39 downto 32));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(39 downto 32));
					elsif rd_pixel_col(2 downto 0) = "101" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(47 downto 40));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(47 downto 40));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(47 downto 40));
					elsif rd_pixel_col(2 downto 0) = "110" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(55 downto 48));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(55 downto 48));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(55 downto 48));
					elsif rd_pixel_col(2 downto 0) = "111" then
						red		<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(63 downto 56));
						green	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(63 downto 56));
						blue	<=	invert_8(pixel_buffer(conv_integer(rd_pixel_col(9 downto 3)))(63 downto 56));
					end if;
				else
					red		<=	(others => '0');
					green	<=	(others => '0');
					blue	<=	(others => '0');
				end if;
			end if;
		end if;
	end process;

	inv_write: for i in 0 to 63 generate
	begin
		inv_wr_val(i)	<=	wr_pixel_val(63 - i);
	end generate;

end behavior;